/* bqd.h - Binary encoding for basic Q data types (C header)
   Copyright (C) 2005  Peter Minten

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <gmp.h>
#include <libq.h>
#include <string.h>
#include <netinet/in.h>

// The initial amount of memory allocated
#define BQD_MEM_INITIAL_SIZE 256
// The extra amount allocated at each expansion
#define BQD_MEM_STEP_SIZE 256

// The memory allocated for the string stream used with gmp stuff
#define BQD_STRING_STREAM_SIZE 1024

// The version of BQD used
#define BQD_VERSION 2

#define CODE_TRUE   0           //type tag
#define CODE_FALSE  1           //type tag
#define CODE_NIL    2           //type tag
#define CODE_INT    3           //type tag, value
#define CODE_UINT   4           //type tag, value
#define CODE_MPZ    5           //type tag, length, value
#define CODE_FLOAT  6           //type tag, value
#define CODE_STRING 7           //type tag, length, value
#define CODE_LIST   8           //type tag, count, elements
#define CODE_TUPLE  9           //type tag, count, elements

// The external interfaces are not declared here due to the FUNCTION magic

// internals
int bytify(expr e, char **v, long *size, long *used);
expr unbytify(char *v, long *pos);

// helpers
inline int require_mem(char **v, long *size, long used, long needed);
