#!/usr/local/bin/q -ctest2
/* smash.q - Marshalling for Q (smash module)
   Copyright (C) 2005, 2006  Peter Minten

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// The smash name comes from a cartoonesk idea, a giant anvil falling on
// an object flattening it so it can be send by mail.

// Smash stores objects with a type tag for unmarshalling. The type tag 
// is added by using a small tuple. For example a dict is represented like: 
// ("Dict", ..dict_content..). The BQD types don't need a type tag: (1) 
// simply means integer 1, note that there is still a tuple around them. 
// For non-baselib types please prefix the typename with "libname/", for 
// example: ("json/JsonException", ...). Note that Tuples are not treated 
// very efficiently (lot's of list<->tuple conversions).

import bqd;
import bqd_old;

// The main external interfaces.
public smash Object, unsmash ByteStr;

// The object (un)marshallers. By adding to these one can add marshalling
// for other data types.
public marshal Object, unmarshal ByteStr;

public type SmashException : Exception = special const smash_exception X;

smash O = bqd_encode (marshal O);
unsmash O = unmarshal (bqd_decode O);

marshal () = ((),);
marshal B:Bool = (B,);
marshal I:Int = (I,);
marshal F:Float = (F,);
marshal S:String = (S,);
marshal L:List = ((map marshal L),);
marshal T:Tuple = ((tuple (map marshal (list T))),);

unmarshal (L:List,) = map unmarshal L;
unmarshal (T:Tuple,) = tuple (map unmarshal (list T));

marshal A:Array = ("Array", map marshal (list A));
unmarshal ("Array",A) = array (map unmarshal A);

marshal B:Bag = ("Bag", map marshal (bag B));
unmarshal ("Bag",B) = bag (map unmarshal B);

marshal D:Dict = ("Dict", pmap marshal (list D));
unmarshal ("Dict",D) = dict (pmap unmarshal D);

marshal H:HDict = ("HDict", pmap marshal (list H));
unmarshal ("HDict",H) = dict (pmap unmarshal H);

marshal H:Heap = ("Heap", map marshal (list H));
unmarshal ("Heap",H) = dict (map unmarshal H);

marshal S:Set = ("Set", map marshal (list S));
unmarshal ("Set",S) = dict (map unmarshal S);

// Default rules
@-1
marshal O = throw (smash_exception "Can't marshal object.") otherwise;
unmarshal (O,) = O otherwise;
@0


pmap F L = map (pmap2 F) L;
pmap2 F (A,B) = (F A, F B);

// Simple test data

test = [1,2,3,-4,-5,(4.1,4.2,4.3),0xABCDEFABCDEF,
        0xABCDEFABCDEF, "a","ba","cba",[1,2,3],(1,2,3)];

test_ = iter 100 id test; //make sure we have enough test data

test1 = write $ bqd_decode $ bqd_encode test_ || writes "\n"; //new version
test2 = write $ bqd_decode2 $ bqd_encode2 test_ || writes "\n"; //old version

test3 = write $ unsmash $ smash $ dict [(1,2),(3,4),(5,6)];

testeq1 = ((bqd_decode $ bqd_encode test) = test);
testeq2 = ((bqd_decode2 $ bqd_encode2 test) = test);